/* Compile with: cc processConfig.c -c -lsocket -lnsl                  */

/* processConfig.c
*
* processConfig( struct confPartnersStruc dest )
*
* This procedure processes the "config.txt" file. The procedure returns a 
* structure with a number of valid conference partners. The maximum number of
* conference partners can be determined by inspecting the value 'dest.size'. 
* The current implementation is restricted to 100. For each conference partner
* there is an structure SocketAddressStruc available. This structure contains
* the mcsap_id, the hostaddress (ipnumber,portno) and the hostname in string 
* representation. Respectivly retrievable by: dest.confPartner[i].mcsap_id, 
* dest.confPartner[i].socketaddress and dest.confPartner[i].hostname!
* 
* *
* Assumptions:
* 1) The hostname string is not longer than 50 characters. 
*    e.g. popper.cs.utwente.nl
* 2) The file contains a number of tripples with format "%d%s%d" and in 
*    between whitespace but no more.
* 3) The first number (%d) must be an integer value except 0. (restriction by
*    specification)
* 4) The second number of each tripple must be an integer value above 1024
*    This is not checked by the procedure! (spec restriction!)
* 5) The number of partners is restricted to a maximum of 100! no checks yet!!!
*/
#include "cp.h"
#include "cfpe.h"
#include "processConfig.h"
#include "debug.h"

int getuseraddr( int *usap, int *srcAddr, struct ConfPartnersStruc *pp)
{
 int refAddr;
 int i;

 *srcAddr = 0;

 if ((*usap = socket(AF_INET, SOCK_DGRAM,0)) <0 ) {
  Error(Init, "Init: Creating socket..");
  return(-1);
 }
 for (refAddr=0;refAddr<pp->size;(refAddr)++) {
  if (bind(*usap, \
      (struct sockaddr *)&pp->Partner[refAddr].socketaddress, \
      sizeof(struct sockaddr_in) \
     )==0)
   break;
  else
   Warning(Init, "Init: Slot id %d is not bindable!", \
	pp->Partner[refAddr].msapid);
 }

 if (refAddr==pp->size) {
  Error(Init, "Init: User could not be bound to socket slot!");
  return(-1);
 }
 Debug(Init, "Init: User receives id '%d':[%s:%d]", \
	pp->Partner[refAddr].msapid, pp->Partner[refAddr].hostname, \
	pp->Partner[refAddr].socketaddress.sin_port);
 Debug(Init, "Init: Maximal %d users conferencing!", pp->size-1);

 *srcAddr = pp->Partner[refAddr].msapid;

 /* eliminating current user as possible partner */
 i = (--pp->size);
 if (refAddr!=i) 
 { /* copy the last possible partner enty onto the current user! */
  memcpy((char *)&pp->Partner[refAddr], (char *)&pp->Partner[i], \
	sizeof(struct SocketAddressStruc));
 }
 return 0;
}

int loadConfig( struct ConfPartnersStruc dest, char *config)
{
 int usap;
 int srcAddr;
 int rvalue;

 if ((rvalue = processConfig(&dest,config))!=0)
 {
  Error(Init, "Init: Error processing Config file...");
  return(rvalue);
 }
 rvalue = getuseraddr(&usap, &srcAddr, &dest);
 if (rvalue != 0) return(rvalue); /* Could not find proper socket address! */
 dest.user.usap = usap;
 dest.user.srcAddr = srcAddr;
 return(0);
}

int processConfig( struct ConfPartnersStruc *dest, char *config)
{
 FILE 	*ifp;
 int 	msapid;
 char	host_name[MAXNAME];
 int	port_number;

 if (strcmp(config,"")==0)  /* no string differences! */
  strcpy(config, CONFIG);

 if ((ifp = fopen(config, "r")) == NULL)
 {
  Error(Init, "Init: opening file: %s\n", CONFIG);
  return(-1);
 }
 else
 { 
  int i=0;
  while ((i<=MAXPARTNERS) \
	&&(fscanf(ifp, "%d%s%d", &msapid, host_name, &port_number) == 3))
  {
   dest->Partner[i].msapid = msapid;
   strcpy(dest->Partner[i].hostname, host_name);
   if(getsocketaddress(host_name,port_number, \
	 &dest->Partner[i].socketaddress)==-1) 
   {
/*	Error(Init, "Init: retrieving socktaddress! -> ignored!"); */
   }
   else 
	i++;
  }
  dest->size = i;
  fclose(ifp);
  if (i==0) /* no entries in config-file! */
   return(-2);
  else
   return(0);
 }
}

int getsocketaddress( char *host_name, int port_number, struct sockaddr_in *addr)
{
 struct hostent		*host;

 if ((host = gethostbyname(host_name)) == 0) 
 {
  Error(Init, "Init: Unknown host: %s", host_name);
  return(-1);
 }
 else
 {
  memcpy((char *) &addr->sin_addr, (char *) host->h_addr, host->h_length);
  addr->sin_family = AF_INET;
  addr->sin_port = htons(port_number);
  return(1);
 }
}



